@@ -3,6 +3,9 @@
 #if __cpp_lib_optional >= 201606L
 #include <optional>
 #endif
+#if SIMDJSON_CPLUSPLUS17
+#include <charconv>
+#endif
 using namespace std;
 using namespace simdjson;
 using error_code=simdjson::error_code;
@@ -150,6 +153,25 @@ simdjson_inline simdjson_result<Car> simdjson::ondemand::document::get() & noexc
 
 #if SIMDJSON_EXCEPTIONS
 
+void main_capture() {
+  padded_string json_padded = "{\"a\":[1,2,3], \"b\": 2, \"c\": \"hello\"}"_padded;
+  std::vector<std::string_view> fields;
+
+  ondemand::parser parser;
+  auto doc = parser.iterate(json_padded);
+  auto object = doc.get_object();
+  for (auto field : object) {
+    fields.push_back(field.value().raw_json());
+  }
+  // Output the fields
+  // Expected output:
+  // [1,2,3]
+  // 2
+  // "hello"
+  for (std::string_view field_ref : fields) {
+    std::cout << field_ref << std::endl;
+  }
+}
 
 int custom_type_on_document() {
   padded_string json = R"( { "make": "Toyota", "model": "Camry",  "year": 2018,
@@ -382,22 +404,84 @@ bool examplecrt_realloc() {
   TEST_SUCCEED();
 }
 
+#if SIMDJSON_CPLUSPLUS17
+bool big_int_array() {
+  TEST_START();
+  ondemand::parser parser;
+  padded_string docdata = R"([-9223372036854775809, 18446744073709551617, 99999999999999999999999 ])"_padded;
+  std::string expected[] = {"-9223372036854775809", "18446744073709551617", "99999999999999999999999 "};
+  ondemand::document doc = parser.iterate(docdata);
+  ondemand::array arr = doc.get_array();
+  size_t i = 0;
+  for(ondemand::value val : arr) {
+    if(i > 3) {
+      std::cerr << "unexpected number of elements" << std::endl;
+      return false;
+    }
+    if(val.get_number_type() != ondemand::number_type::big_integer) {
+      std::cerr << "unexpected number type" << std::endl;
+      std::cout << val.get_number_type() << std::endl;
+      std::cout << val.raw_json_token() << std::endl;
+      return false;
+    }
+    std::string_view token = val.raw_json_token();
+    std::string_view expected_token = expected[i];
+    if(token != expected_token) {
+      std::cerr << "unexpected token: " << token << " expected: " << expected_token << std::endl;
+      return false;
+    }
+    i++;
+  }
+  if(i != 3) {
+    std::cerr << "unexpected number of elements" << std::endl;
+    return false;
+  }
+  TEST_SUCCEED();
+}
+#endif
+
+bool big_int_array_as_double() {
+  TEST_START();
+  ondemand::parser parser;
+  padded_string docdata = R"([-9223372036854775809, 18446744073709551617, 99999999999999999999999 ])"_padded;
+  double dexpected[] = {-9223372036854775808.0, 18446744073709551616.0, 1e23};
+  ondemand::document doc = parser.iterate(docdata);
+  ondemand::array arr = doc.get_array();
+  size_t i = 0;
+  for(ondemand::value val : arr) {
+    if(i > 3) {
+      std::cerr << "unexpected number of elements" << std::endl;
+      return false;
+    }
+    if((val.get_number_type() != ondemand::number_type::big_integer) || (dexpected[i] != val.get_double())) {
+      return false;
+    }
+    i++;
+  }
+  if(i != 3) {
+    std::cerr << "unexpected number of elements" << std::endl;
+    return false;
+  }
+  TEST_SUCCEED();
+}
+
 bool number_tests() {
   TEST_START();
   ondemand::parser parser;
-  padded_string docdata = R"([1.0, 3, 1, 3.1415,-13231232,9999999999999999999])"_padded;
+  padded_string docdata = R"([1.0, 3, 1, 3.1415,-13231232,9999999999999999999,12345678901234567890123])"_padded;
   ondemand::document doc = parser.iterate(docdata);
   ondemand::array arr = doc.get_array();
   for(ondemand::value val : arr) {
     std::cout << val << " ";
     std::cout << "negative: " << val.is_negative() << " ";
     std::cout << "is_integer: " << val.is_integer() << " ";
-    ondemand::number num = val.get_number();
-    ondemand::number_type t = num.get_number_type();
     // direct computation without materializing the number:
     ondemand::number_type dt = val.get_number_type();
-    if(t != dt) { throw std::runtime_error("bug"); }
-    switch(t) {
+    ondemand::number num;
+    auto res = val.get_number().get(num);
+    ondemand::number_type t = num.get_number_type();
+    if(t != dt && res != BIGINT_ERROR) { throw std::runtime_error("bug"); }
+    switch(dt) {
       case ondemand::number_type::signed_integer:
         std::cout  << "integer: " << int64_t(num) << " ";
         std::cout  << "integer: " << num.get_int64() << std::endl;
@@ -410,6 +494,9 @@ bool number_tests() {
         std::cout  << "float: " << double(num) << " ";
         std::cout << "float: " << num.get_double() << std::endl;
         break;
+      case ondemand::number_type::big_integer:
+        std::cout  << "big-integer: " << val.raw_json_token() << std::endl;
+        break;
     }
   }
   TEST_SUCCEED();
@@ -1470,6 +1557,29 @@ bool allow_comma_separated_example() {
   }
   TEST_SUCCEED();
 }
+
+bool issue2215() {
+  TEST_START();
+  ondemand::parser parser;
+  const padded_string json = R"({ "parent": {"child1": {"name": "John"} , "child2": {"name": "Daniel"}} })"_padded;
+  auto doc = parser.iterate(json);
+  ondemand::object parent = doc["parent"];
+  // parent owns the focus
+  ondemand::object c1 = parent["child1"];
+  // c1 owns the focus
+  //
+  std::string_view as1 = c1["name"];
+  // We have that as1 == "John", as long as 'parser' and 'json' live
+  // c2 attempts to grab the focus from parent but fails
+  ondemand::object c2 = parent["child2"];
+  // c2 owns the focus, at this point c1 is invalid
+  std::string_view as2 = c2["name"];
+  // We have that as2 == "Daniel", as long as 'parser' and 'json' live
+  ASSERT_EQUAL(as1, "John");
+  ASSERT_EQUAL(as2, "Daniel");
+  std::cout << as1 << " " << as2 << std::endl;
+  TEST_SUCCEED();
+}
 #endif
 bool test_load_example() {
   TEST_START();
@@ -1778,6 +1888,10 @@ bool value_raw_json_object() {
 bool run() {
   return true
 #if SIMDJSON_EXCEPTIONS
+#if SIMDJSON_CPLUSPLUS17
+    && big_int_array()
+#endif // SIMDJSON_CPLUSPLUS17
+    && big_int_array_as_double()
     && key_raw_json_token()
     && to_optional()
     && value_raw_json_array() && value_raw_json_object()
@@ -1785,7 +1899,7 @@ bool run() {
     && at_end()
     && example1956() && example1958()
     && allow_comma_separated_example()
-//    && basics_1() // Fails because twitter.json isn't in current directory. Compile test only.
+//    && basics_1() // Fails because twitter.json is not in current directory. Compile test only.
     &&  basics_treewalk()
     &&  basics_treewalk_breakline()
     && json_value_with_array_count()
@@ -1827,6 +1941,7 @@ bool run() {
     && current_location_no_error()
     && to_string_example_no_except()
   #if SIMDJSON_EXCEPTIONS
+    && issue2215()
     && to_string_example()
     && raw_string()
     && number_tests()
